#!/usr/bin/python
from pyglet import *
from pyglet.gl import *
from pyglet.window import key, mouse
from pyglet.media import Player as MPlayer
import sys

from platform import Platform
from player import Player
from ninja import Ninja
from util import *
import math
import random
# Would it be more fun if the level was different every time?
#random.seed(0)

window = pyglet.window.Window(width=420, height=560,
                              caption="Roly-Poly Samurai")

window.set_exclusive_mouse(True)

# On Phil's machine, it runs at 1/3 fps without this.
window.set_vsync(0)
# Use this if we ever want to debug events.
#window.push_handlers(pyglet.window.event.WindowEventLogger())

if '-m' not in sys.argv:
    music = pyglet.resource.media('scheming-weasel.wav', streaming=False)
    mplayer = MPlayer()
    mplayer.queue(music)
    mplayer.eos_action = MPlayer.EOS_LOOP
    mplayer.play()


player = Player(210, 280)
ninja = Ninja(None)
score = pyglet.text.Label(text="Score: %6d" % 0, color=(0,0,0,255))
score.x = 15
score.y = -30

glClearColor(1,1,1,1)

glEnable(GL_BLEND)
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

max_tilt = 45

paused = False

highscore = 0
lastscore = 0

def menu():
    global platform
    global player
    global tilt
    global floor
    global lastscore

    tilt = 0
    floor = 0
    
    lastscore = player.score
    
    platform = Platform(210, 180, 200)
    tempp = platform
    player.curPlatform = platform
    player.jumpPlatform = platform

    for i in range(0,3):
        tempp = tempp.makeFollowing()

    ninja.platform = platform.next.next
    ninja.setpos()
    player.reset(210,280)

menu()

def mouse2world(x, y):
    global floor
    return Vec2(x, y+floor)

def update(dt):
    global floor
    global paused
    global platform

    if paused:
        return

    if (player.isDead):
        menu()
    
    player.update(platform, floor)

    global highscore

    if player.score > highscore:
        highscore = player.score

    tempp = platform
    while tempp:
        tempp.update(tilt)
        tempp = tempp.next

    floor = min(floor+2, player.curPlatform.y - 50)

    if player.jumpPlatform.next == ninja.platform:
        ninja.teleport()

    # GC: assume we can't see ones that are too far down.
    if platform != player.curPlatform and \
            platform.next != player.curPlatform and \
            platform.next.next != player.curPlatform:
        platform = platform.next
    
    
    # TODO: add more platforms if we're a certain height above the last one

# Draw event
@window.event
def on_draw():
    global newPlatform
    global newPlatformC
    global newPlatformE
    glLoadIdentity()
    glTranslatef(0, -floor, 0)
    glClear(GL_COLOR_BUFFER_BIT)
    
    # Phil, your hard-coding of 279 is not a good thing! Player _should_ spawn at 0
    glTranslatef(0, 279, 0)
    glBegin(GL_LINES)
    glColor3f(1,0,0)
    glVertex2f(0,highscore)
    glVertex2f(420,highscore)
    glEnd()
    
    score.y = highscore - 30
    score.text = "Highest: %6d" % highscore
    score.draw()
    
    if int(highscore) != int(lastscore):
        glBegin(GL_LINES)
        glColor3f(0,0,1)
        glVertex2f(0,lastscore)
        glVertex2f(420,lastscore)
        glEnd()
        
        if player.score < lastscore:
            score.y = lastscore - 30
            score.text = "Last: %6d" % lastscore
            score.draw()
    
    if int(highscore) != int(player.score):
        glBegin(GL_LINES)
        glColor3f(0,1,0)
        glVertex2f(0,player.score)
        glVertex2f(420,player.score)
        glEnd()
        
        score.y = player.score - 30
        score.text = "Height: %6d" % player.score
        score.draw()
    
    glTranslatef(0, -279, 0)

    tempp = platform
    while tempp:
        tempp.draw()
        tempp = tempp.next
        
    ninja.draw()
    player.draw()

# Mouse move event
@window.event
def on_mouse_motion(x, y, dx, dy):
    global paused
    global tilt
    global max_tilt

    if not paused:
        tilt = clamp(tilt + dx * 0.5, -max_tilt, +max_tilt)

# Mouse drag event
@window.event
def on_mouse_drag(x, y, dx, dy, buttons, modifiers):
    on_mouse_motion(x, y, dx, dy)

# Mouse button down event
@window.event
def on_mouse_press(x, y, button, modifiers):
    global newPlatform
    global newPlatformC
    global newPlatformE
    if button == mouse.LEFT:
        player.startPowerUp()

# Mouse button up event
@window.event
def on_mouse_release(x, y, button, modifiers):
    global tilt
    global floor
    global newPlatform
    global newPlatformC
    global newPlatformE
    if button == mouse.LEFT:
        player.jump()

# Key down event
@window.event
def on_key_press(symbol, modifiers):
    global paused
    if symbol == key.R:
        menu()
    elif symbol == key.P:
        paused = not paused
        

pyglet.clock.schedule_interval(update, 1.0/60.0)

try:
    pyglet.app.run()
except KeyboardInterrupt:
    pass # let me use ctrl-c to quit without giving an error


