from __future__ import division

from pyglet.gl import *
from math import *
from util import *
import random

# Don't allow random choices in orientation to interact with random choices
# for position.
orient = random.Random()

class Platform():
    def __init__(self, x, y, length, tilt=0, prev=None):
        self.x = x
        self.y = y
        self.length = length
        self.width = 10
        self.tilt = tilt
        self.offset_tilt = tilt
        self.next = None
        self.prev = prev

        # Three images are too slow for me.
        #self.image_l = pyglet.resource.image('bamboo-left.png').get_texture()
        #self.image_m = pyglet.resource.image('bamboo-middle.png').get_texture()
        #self.image_r = pyglet.resource.image('bamboo-right.png').get_texture()
        
        self.image = pyglet.resource.image('platform.png')

        self.scale_x = length * 0.5 * orient.choice([-1, 1])
        self.scale_y = 35 * 0.5 * orient.choice([-1, 1])

    def getEndpoints(self):
        c = cos(radians(self.tilt))
        s = sin(radians(self.tilt))
        h = self.length/2.0
        offset = Vec2(c*h, s*h)
        pos = Vec2(self.x, self.y)
        return (pos-offset, pos+offset)
	
    def update(self, world_tilt):
        self.tilt = self.offset_tilt + world_tilt
	
    def draw(self):
        
        glPushMatrix()
        glTranslatef(self.x, self.y, 0)
        glRotatef(self.tilt, 0, 0, 1)
        glScalef(self.scale_x, self.scale_y, 1)

        glColor3f(1,1,1)

        self.image.blit(-1, -1, width=2, height=2)

        #w_l = self.image_l.width
        #w_m = self.image_m.width
        #w_r = self.image_r.width
        #w_t = w_l + w_r + w_m
        
       # self.image.blit(-self.image.width * 0.5, -self.image.height * 0.5, width=self.image.width, height=self.image.height)

        #self.image_l.blit(-1, -1, width=w_l/w_t, height=2)
        #self.image_m.blit(-1 + w_l/w_t, -1, width = 2 - (w_l + w_r)/w_t, height=2)
        #self.image_r.blit(1 - w_r/w_t, -1, width = w_r/w_t, height=2)
        glPopMatrix()


    # If the prior is near one side of the screen, the new one goes towards the
    # middle. Otherwise, we go left or right randomly. The new one leaves at
    # least 50px on the far end of the previous one uncovored, and has a gap of
    # no more than 50px from the near end.

    # We assume an angle of 0 on both platforms for all of these calculations,
    # and then pick a random offset tilt. Height and offset tilt are chosen
    # independently of x-position and length.
    def makeFollowing(self):
        left = self.x - self.width/2
        right = self.x + self.width/2
        q = range(0, 420+1, 420//4) # range excludes its right endpoint.

        if q[0] < left <= q[1]:
            goLeft = False
        elif q[3] < right <= q[4]:
            goLeft = True
        else:
            goLeft = random.choice([True, False])

        if goLeft:
            newright = random.uniform(left-50, right-50)
            newlength = random.uniform(100, min(200, newright))
            newx = newright - newlength/2
        else:
            newleft = random.uniform(left+50, right+50)
            newlength = random.uniform(100, min(200, 420-newleft))
            newx = newleft + newlength/2

        newy = self.y + random.uniform(150, 200)
        newtilt = random.uniform(-20, 20)
        
        new = Platform(newx, newy, newlength, tilt=newtilt,
                       prev = self)

        self.next = new

        return new
