from __future__ import division
# Don't import * to avoid namespace pollution
import math

def clamp(value, low, high):
    return max(low, min(high, value))

class Vec2:
    __slots__ = ['x', 'y']

    def __init__(self, x, y):
        self.x = x
        self.y = y
    
    def __add__(self, other):
        return Vec2(self.x+other.x, self.y+other.y)
    
    def __sub__(self, other):
        return Vec2(self.x-other.x, self.y-other.y)
    
    def __mul__(self, other):
        return Vec2(self.x*other, self.y*other)
    
    __rmul__ = __mul__

    def __div__(self, other):
        return self * (1/other)

    __truediv__ = __div__

    def __eq__(self, other):
        return self.x == other.x and self.y == other.y

    def __str__(self):
        return "(%s, %s)" % (self.x, self.y)

    def __repr__(self):
        return str(self)
    
    def dot(self, other):
        return self.x*other.x + self.y*other.y

    def length(self):
        return math.sqrt(self.dot(self))

    def normalize(self):
        if self.x == 0 and self.y == 0:
            return self
        else:
            return self/self.length()

    # Returns the closest point to self on the line segment [a, b].
    def closestPoint(self, a, b):
        if a == b:
            return a
        v = b - a
        w = self - a
        t = clamp(w.dot(v) / v.dot(v), 0, 1)
        return a + v*t
